local RC = LibStub("LibRaidComm-1.0")
local classKR = { "전사", "드루이드", "성기사", "사제", "주술사", "도적", "마법사", "흑마법사", "사냥꾼", "죽음의 기사" }
local classToID = { ["전사"] = 1, ["드루이드"] = 2, ["성기사"] = 3, ["사제"] = 4, ["주술사"] = 5, ["도적"] = 6, ["마법사"] = 7, ["흑마법사"] = 8, ["사냥꾼"] = 9, ["죽음의 기사"] = 10 }
local points = { "TopLeft", "TopRight", "BottomLeft", "BottomRight" }
local baseItemIcons = { "Head", "Neck", "Shoulder", "Shirt", "Chest", "Waist", "Legs", "Feet", "Wrists", "Hands", "Finger", "Finger", "Trinket", "Trinket", "Chest", "MainHand", "SecondaryHand", "Ranged", "Tabard" }
local relicClass = { ["드루이드"] = 1, ["주술사"] = 1, ["성기사"] = 1, ["죽음의 기사"] = 1 }
local SWClass = { ["전사"] = 1, ["도적"] = 1, ["죽음의 기사"] = 1 }
local icon, item, talentIcons, talentArrows, talentBranchs, maxTier, background, prereqs, name, rank, tier, column, dtier, dcolumn, n, maxLevel, b1, b2, talentDB
local requestUpdater = CreateFrame("Frame")

local unpack = _G.unpack
local strfind = _G.strfind
local strmatch = _G.strmatch
local strsplit = _G.strsplit
local tonumber = _G.tonumber
local tinsert = _G.tinsert
local format = _G.format
local max = _G.max
local strlower = _G.strlower
local IsAltKeyDown = _G.IsAltKeyDown
local IsControlKeyDown = _G.IsControlKeyDown
local IsShiftKeyDown = _G.IsShiftKeyDown
local IsModifiedClick = _G.IsModifiedClick
local GetItemInfo = _G.GetItemInfo
local GetItemIcon = _G.GetItemIcon
local GetItemQualityColor = _G.GetItemQualityColor
local GetTime = _G.GetTime

function InvenArmory:SetRequestUpdater(method, sender)
	requestUpdater.method = method
	requestUpdater.sender = sender
	requestUpdater.timer = GetTime() + 0.2
	requestUpdater:SetScript("OnUpdate", InvenArmory.RequestUpdater)
end

function InvenArmory:RequestUpdater()
	if self.method and self.sender and self.timer then
		if GetTime() >= self.timer then
			RC[self.method](RC, self.sender)
			InvenArmory:ClearRequestUpdater()
		end
	else
		InvenArmory:ClearRequestUpdater()
	end
end

function InvenArmory:ClearRequestUpdater()
	requestUpdater:SetScript("OnUpdate", nil)
	requestUpdater.method = nil
	requestUpdater.sender = nil
	requestUpdater.timer = nil
end

function InvenArmory:Inspect(event, sender, items)
	if self.inspectName ~= sender then return end
	self.inspectFail = nil
	if InvenArmoryFrame then
		if InvenArmoryFrame:IsShown() then
			PlaySound("igCharacterInfoOpen")
		else
			ShowUIPanel(InvenArmoryFrame)
		end
	else
		self:CreateFrame()
		ShowUIPanel(InvenArmoryFrame)
	end
	if self.userDB[sender] == nil then
		self.userDB[sender] = { timer = GetTime() + 120, items = items }
	end
	InvenArmoryFrame.statDraw = nil
	InvenArmoryFrame.db = { name = sender, class = nil, index = nil, talentDB = nil }
	for i = 1, #items do
		item = items[i] ~= "*" and items[i] or nil
		if item then
			icon = GetItemIcon(items[i])
			quality = select(3, GetItemInfo(items[i]))
		elseif i == 18 and relicClass[InvenArmoryFrame.class] then
			icon = "Interface\\Paperdoll\\UI-PaperDoll-Slot-Relic.blp"
			quality = nil
		else
			icon = "Interface\\Paperdoll\\UI-PaperDoll-Slot-"..baseItemIcons[i]..".blp"
			quality = nil
		end
		InvenArmoryFrame.items[i].link = item
		SetItemButtonTexture(InvenArmoryFrame.items[i], icon)
		if quality then
			InvenArmoryFrame.items[i].qcolor:SetVertexColor(GetItemQualityColor(quality))
			InvenArmoryFrame.items[i].qcolor:Show()
		else
			InvenArmoryFrame.items[i].qcolor:Hide()
		end
	end
	InvenArmoryFrame.name:SetText(InvenArmoryFrame.db.name)
	self.inspectName = nil
	for i = 1, 4 do
		InvenArmoryFrame.buttons[i]:Hide()
	end
	self:SetTalent()
	if self.requestPlayer then
		self:SetRequestUpdater("Talent", sender)
	end
end

function InvenArmory:Talent(event, sender, talent)
	if not InvenArmoryFrame then return end
	if InvenArmoryFrame.db.name ~= sender then return end
	if self.userDB[sender] then
		self.userDB[sender].talent = talent
	end
	talentDB = { strsplit("#", talent) }
	InvenArmoryFrame.db.index = classToID[talentDB[1]]
	InvenArmoryFrame.db.class = talentDB[1]
	InvenArmoryFrame.db.talentDB = {}
	InvenArmoryFrame.db.talentPoints = { 0, 0, 0 }
	InvenArmoryFrame.db.talentMax = 1
	for i = 1, 3 do
		n = strlen(talentDB[i + 1])
		if n ~= #InvenArmoryTalentDB[InvenArmoryFrame.db.index][i] then
			if self.requestPlayer then
				self:SetRequestUpdater("Stat", sender)
			end
			return
		end
		InvenArmoryFrame.buttons[i]:Show()
		InvenArmoryFrame.db.talentDB[i] = {}
		for j = 1, n do
			rank = tonumber(strsub(talentDB[i + 1], j, j))
			name, icon, tier, column, link = strsplit("#", InvenArmoryTalentDB[InvenArmoryFrame.db.index][i][j])
			tinsert(InvenArmoryFrame.db.talentDB[i], { name, "Interface\\Icons\\"..icon, tonumber(tier), tonumber(column), rank, link })
			InvenArmoryFrame.db.talentPoints[i] = InvenArmoryFrame.db.talentPoints[i] + rank
			if i > 1 and InvenArmoryFrame.db.talentPoints[i] > InvenArmoryFrame.db.talentPoints[InvenArmoryFrame.db.talentMax] then
				InvenArmoryFrame.db.talentMax = i
			end
		end
		InvenArmoryFrame.buttons[i]:SetText(InvenArmoryTalentDB[InvenArmoryFrame.db.index][i].name.."("..InvenArmoryFrame.db.talentPoints[i]..")")
	end
	InvenArmoryFrame.name:SetText(InvenArmoryFrame.db.name.." ("..InvenArmoryFrame.db.class..")")
	self:SetTalent(InvenArmoryFrame.db.talentMax)
	if self.requestPlayer then
		self:SetRequestUpdater("Stat", sender)
	end
end

function InvenArmory:Stat(event, sender, ...)
	if not InvenArmoryFrame then return end
	if InvenArmoryFrame.db.name == sender then
		if self.userDB[sender] then
			self.userDB[sender].stat = { ... }
		end
		InvenArmoryFrame.buttons[4].isOK = true
		InvenArmoryFrame.buttons[4]:Show()
		InvenArmoryFrame.db.stat = { ... }
		if InvenArmoryFrame.db.talentDB == nil then
			self:SetTalent(4)
		end
	end
	if self.requestPlayer then
		self:SetRequestUpdater("Info", sender)
	end
end

function InvenArmory:Info(event, sender, level, class, race)
	if not InvenArmoryFrame then return end
	if InvenArmoryFrame.db.name == sender then
		if self.userDB[sender] then
			self.userDB[sender].level = level
			self.userDB[sender].class = class
			self.userDB[sender].race = race
		end
		InvenArmoryFrame.name:SetFormattedText("%s (레벨 %d %s %s)", InvenArmoryFrame.db.name, level, race, class)
	end
end

function InvenArmory:SetTalent(index)
	InvenArmoryFrame.db.talentTab = index
	if index and InvenArmoryTalentDB[InvenArmoryFrame.db.index] and InvenArmoryTalentDB[InvenArmoryFrame.db.index][index] and InvenArmoryTalentDB[InvenArmoryFrame.db.index][index].bg then
		background = "Interface\\TalentFrame\\"..InvenArmoryTalentDB[InvenArmoryFrame.db.index][index].bg.."-"
	else
		background = "Interface\\TalentFrame\\MageFire-"
	end
	for i = 1, 4 do
		InvenArmoryFrame["Talent"..points[i]]:SetTexture(background..points[i])
	end
	if index then
		for i = 1, 4 do
			if i == index then
				InvenArmoryFrame.buttons[i]:GetNormalTexture():SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
			else
				InvenArmoryFrame.buttons[i]:GetNormalTexture():SetTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
			end
		end
	end
	if index and InvenArmoryFrame.db.talentDB and InvenArmoryFrame.db.talentDB[index] then
		InvenArmoryFrame.scrollframe:Show()
		InvenArmoryFrame.scrollchild:SetScale(0.9)
		if InvenArmoryFrame.statFrame then
			InvenArmoryFrame.statFrame:Hide()
		end
	elseif index and InvenArmoryFrame.buttons[4].isOK then
		InvenArmoryFrame.scrollframe:Show()
		InvenArmoryFrame.scrollchild:SetScale(1)
		for i = 1, #talentIcons do
			talentIcons[i]:Hide()
		end
		for i = 1, #talentArrows do
			talentArrows[i]:Hide()
		end
		for i = 1, #talentBranchs do
			talentBranchs[i]:Hide()
		end
		InvenArmory:SetStat()
		return
	else
		InvenArmoryFrame.scrollframe:Hide()
		for i = 1, 3 do
			InvenArmoryFrame.buttons[i]:Hide()
		end
		return
	end
	InvenArmoryFrame.db.talentLocation = {}
	InvenArmoryFrame.db.talentRank = {}
	maxTier, maxLevel = 0, 0
	for i = 1, #InvenArmoryFrame.db.talentDB[index] do
		if not talentIcons[i] then
			talentIcons[i] = CreateFrame("Button", "InvenArmoryFrameTalent"..i, InvenArmoryFrame.scrollchild, "TalentButtonTemplate")
			talentIcons[i]:SetID(i)
			talentIcons[i].texture = _G["InvenArmoryFrameTalent"..i.."IconTexture"]
			talentIcons[i].rank = _G["InvenArmoryFrameTalent"..i.."Rank"]
			talentIcons[i].rankBorder = _G["InvenArmoryFrameTalent"..i.."RankBorder"]
			talentIcons[i].slot = _G["InvenArmoryFrameTalent"..i.."Slot"]
			talentIcons[i]:EnableDrawLayer("BORDER")
			talentIcons[i]:SetScript("OnEnter", self.ButtonOnEnter)
			talentIcons[i]:SetScript("OnLeave", self.ButtonOnLeave)
			talentIcons[i]:SetScript("OnClick", self.ButtonOnClick)
			talentIcons[i]:EnableMouse(true)
		end
		InvenArmoryFrame.db.talentLocation[InvenArmoryFrame.db.talentDB[index][i][3]] = InvenArmoryFrame.db.talentLocation[InvenArmoryFrame.db.talentDB[index][i][3]] or {}
		InvenArmoryFrame.db.talentLocation[InvenArmoryFrame.db.talentDB[index][i][3]][InvenArmoryFrame.db.talentDB[index][i][4]] = talentIcons[i]
		InvenArmoryFrame.db.talentRank[InvenArmoryFrame.db.talentDB[index][i][3]] = InvenArmoryFrame.db.talentRank[InvenArmoryFrame.db.talentDB[index][i][3]] or {}
		InvenArmoryFrame.db.talentRank[InvenArmoryFrame.db.talentDB[index][i][3]][InvenArmoryFrame.db.talentDB[index][i][4]] = InvenArmoryFrame.db.talentDB[index][i][5] or 0
		maxTier = max(maxTier, InvenArmoryFrame.db.talentDB[index][i][3])
		talentIcons[i]:ClearAllPoints()
		talentIcons[i]:SetPoint("TOPLEFT", InvenArmoryFrame.scrollchild, "TOPLEFT", 5 + (InvenArmoryFrame.db.talentDB[index][i][4] - 1) * 57, -(InvenArmoryFrame.db.talentDB[index][i][3] - 1) * 57 - 5)
		talentIcons[i].texture:SetTexture(InvenArmoryFrame.db.talentDB[index][i][2])
		if InvenArmoryFrame.db.talentDB[index][i][5] > 0 then
			SetItemButtonDesaturated(talentIcons[i], nil)
			talentIcons[i].rank:SetText(InvenArmoryFrame.db.talentDB[index][i][5])
			talentIcons[i].slot:SetVertexColor(0.1, 1.0, 0.1)
			talentIcons[i].rank:Show()
			talentIcons[i].rankBorder:Show()
		else
			SetItemButtonDesaturated(talentIcons[i], 1, 0.65, 0.65, 0.65)
			talentIcons[i].slot:SetVertexColor(0.5, 0.5, 0.5)
			talentIcons[i].rank:Hide()
			talentIcons[i].rankBorder:Hide()
		end
		talentIcons[i].link = format("|cff4e96f7|Htalent:%d:%d|h[%s]|h|r", InvenArmoryFrame.db.talentDB[index][i][6], InvenArmoryFrame.db.talentDB[index][i][5] - 1, InvenArmoryFrame.db.talentDB[index][i][1])
		talentIcons[i]:Show()
	end
	for i = #InvenArmoryFrame.db.talentDB[index] + 1, 40 do
		talentIcons[i].link = nil
		talentIcons[i]:Hide()
	end
	if background ~= "Interface\\TalentFrame\\MageFire-" and InvenArmoryTalentDB[InvenArmoryFrame.db.index][index].prereqs then
		n = 0
		prereqs = { strsplit("#", InvenArmoryTalentDB[InvenArmoryFrame.db.index][index].prereqs) }
		for i = 1, #prereqs do
			tier, column, dtier, dcolumn = strmatch(prereqs[i], "(%d+):(%d+)|(%d+):(%d+)")
			tier, column, dtier, dcolumn = tonumber(tier), tonumber(column), tonumber(dtier), tonumber(dcolumn)
			if column == dcolumn and tier > dtier then
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0, 0, 0, 0.484375, 0.125, 0, 0.125, 0.484375)
				talentBranchs[n]:SetPoint("TOP", InvenArmoryFrame.db.talentLocation[dtier][dcolumn], "BOTTOM", -0.5, 0)
				talentBranchs[n]:SetPoint("BOTTOM", InvenArmoryFrame.db.talentLocation[tier][column], "TOP", -0.5, 0)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
			elseif tier == dtier and column > dcolumn then
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0.2578125, 0, 0.2578125, 0.5, 0.3828125, 0, 0.3828125, 0.5)
				talentBranchs[n]:SetPoint("RIGHT", InvenArmoryFrame.db.talentLocation[tier][column], "LEFT", 0, 0)
				talentBranchs[n]:SetPoint("LEFT", InvenArmoryFrame.db.talentLocation[dtier][dcolumn], "RIGHT", 0, 0)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
			elseif tier == dtier and column < dcolumn then
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0.2578125, 0, 0.2578125, 0.5, 0.3828125, 0, 0.3828125, 0.5)
				talentBranchs[n]:SetPoint("RIGHT", InvenArmoryFrame.db.talentLocation[dtier][dcolumn], "LEFT", 0, 0)
				talentBranchs[n]:SetPoint("LEFT", InvenArmoryFrame.db.talentLocation[tier][column], "RIGHT", 0, 0)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
			elseif tier > dtier and column > dcolumn then
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0.2578125, 0, 0.2578125, 0.5, 0.3828125, 0, 0.3828125, 0.5)
				talentBranchs[n]:SetPoint("LEFT", InvenArmoryFrame.db.talentLocation[dtier][dcolumn], "RIGHT", 0, 0)
				talentBranchs[n]:SetWidth((column - dcolumn) * 57 - 34 - 0.5)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0.515625, 0, 0.515625, 0.5, 0.640625, 0, 0.640625, 0.5)
				talentBranchs[n]:SetPoint("LEFT", talentBranchs[n - 1], "RIGHT", 0, 0)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0, 0, 0, 0.484375, 0.125, 0, 0.125, 0.484375)
				talentBranchs[n]:SetPoint("BOTTOM", InvenArmoryFrame.db.talentLocation[tier][column], "TOP", 0, 0)
				talentBranchs[n]:SetHeight((tier - dtier) * 57 - 34 + 1)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
			elseif tier > dtier and column < dcolumn then
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0.2578125, 0, 0.2578125, 0.5, 0.3828125, 0, 0.3828125, 0.5)
				talentBranchs[n]:SetPoint("RIGHT", InvenArmoryFrame.db.talentLocation[dtier][dcolumn], "LEFT", 0, 0)
				talentBranchs[n]:SetWidth((dcolumn - column) * 57 - 34 - 0.5)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0.640625, 0, 0.640625, 0.5, 0.515625, 0, 0.515625, 0.5)
				talentBranchs[n]:SetPoint("RIGHT", talentBranchs[n - 1], "LEFT", 0, 0)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
				n = self:CreateBranch(n)
				talentBranchs[n]:SetTexCoord(0, 0, 0, 0.484375, 0.125, 0, 0.125, 0.484375)
				talentBranchs[n]:SetPoint("BOTTOM", InvenArmoryFrame.db.talentLocation[tier][column], "TOP", 0, 0)
				talentBranchs[n]:SetHeight((tier - dtier) * 57 - 34 + 1)
				self:SetArrowColor(talentBranchs[n], tier, column, dtier, dcolumn)
			end
			if talentArrows[i] then
				talentArrows[i]:ClearAllPoints()
			else
				talentArrows[i] = talentArrows[0]:CreateTexture(nil, "OVERLAY")
				talentArrows[i]:SetTexture("Interface\\TalentFrame\\UI-TalentArrows")
				talentArrows[i]:SetWidth(32)
				talentArrows[i]:SetHeight(32)
			end
			if tier > dtier then
				talentArrows[i]:SetTexCoord(0, 0, 0, 0.5, 0.5, 0, 0.5, 0.5)
				talentArrows[i]:SetPoint("BOTTOM", InvenArmoryFrame.db.talentLocation[tier][column], "TOP", 0, -14)
				self:SetArrowColor(talentArrows[i], tier, column, dtier, dcolumn)
			elseif  column > dcolumn then
				talentArrows[i]:SetTexCoord(0.5, 0, 0.5, 0.5, 1, 0, 1, 0.5)
				talentArrows[i]:SetPoint("RIGHT", InvenArmoryFrame.db.talentLocation[tier][column], "LEFT", 14, 0)
				self:SetArrowColor(talentArrows[i], tier, column, dtier, dcolumn)
			elseif column < dcolumn then
				talentArrows[i]:SetTexCoord(1, 0, 1, 0.5, 0.5, 0, 0.5, 0.5)
				talentArrows[i]:SetPoint("LEFT", InvenArmoryFrame.db.talentLocation[tier][column], "RIGHT", -14, 0)
				self:SetArrowColor(talentArrows[i], tier, column, dtier, dcolumn)
			end
		end
		for i = #prereqs + 1, #talentArrows do
			talentArrows[i]:Hide()
		end
		for i = n + 1, #talentBranchs do
			talentBranchs[i]:Hide()
		end
	end
	InvenArmoryFrame.scrollchild:SetHeight((maxTier - 1) * 65)
	InvenArmoryFrame.scrollframe:Show()
	if talentArrows and talentArrows[0] then
		talentArrows[0]:Show()
	end
end

function InvenArmory:CreateBranch(n)
	n = n + 1
	if talentBranchs[n] then
		talentBranchs[n]:ClearAllPoints()
	else
		talentBranchs[n] = InvenArmoryFrame.scrollchild:CreateTexture(nil, "ARTWORK")
		talentBranchs[n]:SetTexture("Interface\\TalentFrame\\UI-TalentBranches")
	end
	talentBranchs[n]:SetWidth(32)
	talentBranchs[n]:SetHeight(32)
	return n
end

function InvenArmory:SetArrowColor(texture, tier, column, dtier, dcolumn)
	if texture and tier and column and dtier and dcolumn and InvenArmoryFrame.db.talentRank[tier][column] and InvenArmoryFrame.db.talentRank[dtier][dcolumn] then
		if InvenArmoryFrame.db.talentRank[tier][column] > 0 and InvenArmoryFrame.db.talentRank[dtier][dcolumn] > 0 then
			texture:SetDesaturated(nil)
		else
			texture:SetDesaturated(1)
		end
		texture:Show()
	else
		texture:Hide()
	end
end

function InvenArmory:SetStat()
	if InvenArmoryFrame.statDraw == nil then
		if InvenArmoryFrame.statFrame == nil then
			InvenArmoryFrame.statFrame = CreateFrame("Frame", nil, InvenArmoryFrame.scrollchild)
			InvenArmoryFrame.statFrame:SetAllPoints(InvenArmoryFrame.scrollchild)
			InvenArmoryFrame.statFrame.list = {}
			local anchor = InvenArmoryFrame.statFrame
			local list
			local function CreateStatList(title, content)
				list = InvenArmoryFrame.statFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
				list:SetText("["..title.."]")
				list:SetHeight(18)
				list:SetJustifyH("LEFT")
				list:SetTextColor(1, 1, 1)
				if anchor == InvenArmoryFrame.statFrame then
					list:SetPoint("TOPLEFT", anchor, "TOPLEFT")
				else
					list:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -5)
				end
				anchor = list
				for i = 1, #content do
					list = InvenArmoryFrame.statFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
					list:SetText(content[i]..":")
					list:SetWidth(78)
					list:SetHeight(18)
					list:SetJustifyH("LEFT")
					list:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT")
					anchor = list
					list = InvenArmoryFrame.statFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
					list:SetWidth(120)
					list:SetHeight(18)
					list:SetJustifyH("RIGHT")
					list:SetTextColor(1, 1, 1)
					list:SetPoint("LEFT", anchor, "RIGHT")
					tinsert(InvenArmoryFrame.statFrame.list, list)
				end
			end
			CreateStatList("기본", { "최대 생명력", "최대 마나" })
			CreateStatList("기본 능력치", { "힘", "민첩성", "체력", "지능", "정신력", "방어도" })
			CreateStatList("근접", { "공격력", "속도", "전투력", "적중도", "치명타율", "숙련" })
			CreateStatList("원거리", { "공격력", "속도", "전투력", "적중도", "치명타율" })
			CreateStatList("주문", { "공격력 증가", "치유량 증가", "적중도", "극대화율", "공격 가속도", "마나 회복량" })
			CreateStatList("방어", { "방어도", "방어 숙련", "회피율", "무기 막기", "방패 막기", "탄력도" })
		end
		for i = 1, #InvenArmoryFrame.statFrame.list do
			InvenArmoryFrame.statFrame.list[i]:SetText(InvenArmoryFrame.db.stat[i])
		end
		InvenArmoryFrame.scrollchild:SetHeight(617)
		InvenArmoryFrame.statDraw = true
	end
	InvenArmoryFrame.statFrame:Show()
end

function InvenArmory:CreateFrame()
	local f = CreateFrame("Frame", "InvenArmoryFrame", UIParent)
	f:SetFrameStrata("MEDIUM")
	f:SetToplevel(true)
	f:SetWidth(384)
	f:SetHeight(512)
	f:SetAttribute("UIPanelLayout-defined", true)
	f:SetAttribute("UIPanelLayout-enabled", true)
	f:SetAttribute("UIPanelLayout-whileDead", true)
	f:SetAttribute("UIPanelLayout-area", "left")
	f:SetAttribute("UIPanelLayout-pushable", 3)
	f:SetHitRectInsets(0, 35, 0, 75)
	f:SetPoint("CENTER", UIParent, "CENTER")
	f:EnableMouse(true)
	f:SetScript("OnShow", function() PlaySound("igCharacterInfoOpen") end)
	f:SetScript("OnHide", function() PlaySound("igCharacterInfoClose") end)
	local t = f:CreateTexture(nil, "BORDER")
	t:SetTexture("Interface\\TaxiFrame\\UI-TaxiFrame-TopLeft")
	t:SetWidth(256)
	t:SetHeight(256)
	t:SetPoint("TOPLEFT", f, "TOPLEFT")
	t = f:CreateTexture(nil, "BORDER")
	t:SetTexture("Interface\\TaxiFrame\\UI-TaxiFrame-TopRight")
	t:SetWidth(128)
	t:SetHeight(256)
	t:SetPoint("TOPRIGHT", f, "TOPRIGHT")
	t = f:CreateTexture(nil, "BORDER")
	t:SetTexture("Interface\\TaxiFrame\\UI-TaxiFrame-BotLeft")
	t:SetWidth(256)
	t:SetHeight(256)
	t:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT")
	t = f:CreateTexture(nil, "BORDER")
	t:SetTexture("Interface\\TaxiFrame\\UI-TaxiFrame-BotRight")
	t:SetWidth(128)
	t:SetHeight(256)
	t:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT")
	t = CreateFrame("Button", nil, f, "UIPanelCloseButton")
	t:SetPoint("TOPRIGHT", f, "TOPRIGHT", -30, -8)
	t = f:CreateTexture(nil, "BACKGROUND")
	t:SetWidth(64)
	t:SetHeight(64)
	t:SetPoint("TOPLEFT", f, "TOPLEFT", 6, -6)
	SetPortraitToTexture(t, "Interface\\Icons\\Ability_Stealth")
	t = f:CreateFontString(nil, "BORDER", "GameFontHighlight")
	t:SetWidth(300)
	t:SetHeight(14)
	t:SetPoint("TOP", f, "TOP", 0, -16)
	t:SetJustifyH("CENTER")
	t:SetText("인벤 전투정보실 v"..self.version)
	f.name = f:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	f.name:SetWidth(300)
	f.name:SetHeight(14)
	f.name:SetJustifyH("CENTER")
	f.name:SetPoint("TOP", t, "BOTTOM", 0, -5)
	local ap1, ap2 = 1.059, 1.07
	f.TalentTopLeft = f:CreateTexture(nil, "ARTWORK")
	f.TalentTopLeft:SetWidth(256 * ap1)
	f.TalentTopLeft:SetHeight(256 * ap2)
	f.TalentTopLeft:SetPoint("TOPLEFT", f, "TOPLEFT", 21, -75)
	f.TalentTopRight = f:CreateTexture(nil, "ARTWORK")
	f.TalentTopRight:SetWidth(64 * ap1)
	f.TalentTopRight:SetHeight(256 * ap2)
	f.TalentTopRight:SetPoint("TOPLEFT", f.TalentTopLeft, "TOPRIGHT")
	f.TalentBottomLeft = f:CreateTexture(nil, "ARTWORK")
	f.TalentBottomLeft:SetWidth(256 * ap1)
	f.TalentBottomLeft:SetHeight(128 * ap2)
	f.TalentBottomLeft:SetPoint("TOPLEFT", f.TalentTopLeft, "BOTTOMLEFT")
	f.TalentBottomRight = f:CreateTexture(nil, "ARTWORK")
	f.TalentBottomRight:SetWidth(64 * ap1)
	f.TalentBottomRight:SetHeight(128 * ap2)
	f.TalentBottomRight:SetPoint("TOPLEFT", f.TalentTopLeft, "BOTTOMRIGHT")
	f.items = {}
	for i = 1, 19 do
		f.items[i] = CreateFrame("Button", "InvenArmoryItemSlot"..i, f, "ItemButtonTemplate")
		f.items[i]:SetID(i)
		f.items[i].texture = _G[f.items[i]:GetName().."IconTexture"]
		f.items[i].qcolor = f.items[i]:CreateTexture(nil, "OVERLAY")
		f.items[i].qcolor:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
		f.items[i].qcolor:SetBlendMode("ADD")
		f.items[i].qcolor:SetHeight(64)
		f.items[i].qcolor:SetWidth(64)
		f.items[i].qcolor:SetPoint("CENTER", f.items[i], "CENTER", 0, 1)
		f.items[i].qcolor:Hide()
		f.items[i]:SetScript("OnEnter", self.ButtonOnEnter)
		f.items[i]:SetScript("OnLeave", self.ButtonOnLeave)
		f.items[i]:SetScript("OnClick", self.ButtonOnClick)

	end
	f.items[1]:SetPoint("TOPLEFT", f.TalentTopLeft, "TOPLEFT", 6, -5)
	f.items[2]:SetPoint("TOP", f.items[1], "BOTTOM", 0, -6.5)
	f.items[3]:SetPoint("TOP", f.items[2], "BOTTOM", 0, -6.5)
	f.items[15]:SetPoint("TOP", f.items[3], "BOTTOM", 0, -6.5)
	f.items[5]:SetPoint("TOP", f.items[15], "BOTTOM", 0, -6.5)
	f.items[4]:SetPoint("TOP", f.items[5], "BOTTOM", 0, -6.5)
	f.items[19]:SetPoint("TOP", f.items[4], "BOTTOM", 0, -6.5)
	f.items[9]:SetPoint("TOP", f.items[19], "BOTTOM", 0, -6.5)
	f.items[10]:SetPoint("TOPRIGHT", f.TalentTopRight, "TOPRIGHT", -28, -5)
	f.items[6]:SetPoint("TOP", f.items[10], "BOTTOM", 0, -6.5)
	f.items[7]:SetPoint("TOP", f.items[6], "BOTTOM", 0, -6.5)
	f.items[8]:SetPoint("TOP", f.items[7], "BOTTOM", 0, -6.5)
	f.items[11]:SetPoint("TOP", f.items[8], "BOTTOM", 0, -6.5)
	f.items[12]:SetPoint("TOP", f.items[11], "BOTTOM", 0, -6.5)
	f.items[13]:SetPoint("TOP", f.items[12], "BOTTOM", 0, -6.5)
	f.items[14]:SetPoint("TOP", f.items[13], "BOTTOM", 0, -6.5)
	f.items[17]:SetPoint("BOTTOM", f, "BOTTOM", -12, 90)
	f.items[16]:SetPoint("RIGHT", f.items[17], "LEFT", -6.5, 0)
	f.items[18]:SetPoint("LEFT", f.items[17], "RIGHT", 6.5, 0)
	talentIcons = {}
	f.buttons = {}
	for i = 1, 4 do
		f.buttons[i] = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
		f.buttons[i]:SetID(i)
		f.buttons[i]:SetWidth(65)
		f.buttons[i]:SetHeight(21)
		f.buttons[i]:RegisterForClicks("AnyUp")
		f.buttons[i]:SetScript("OnClick", self.TalentButtonOnClick)
	end
	f.buttons[2]:SetPoint("TOP", f.name, "BOTTOM", -14, 0)
	f.buttons[1]:SetPoint("RIGHT", f.buttons[2], "LEFT", -2, 0)
	f.buttons[3]:SetPoint("LEFT", f.buttons[2], "RIGHT", 2, 0)
	f.buttons[4]:SetPoint("LEFT", f.buttons[3], "RIGHT", 2, 0)
	f.buttons[4]:SetText("스텟")
	f:SetScript("OnHide", function(self)
		self.db = {}
		for i = 1, 4 do
			self.buttons[i]:Hide()
		end
		for i = 1, 4 do
			self["Talent"..points[i]]:SetTexture()
		end
		for i = 1, 19 do
			self.items[i].link = nil
		end
		self.buttons[4].isOK = nil
		self.statDraw = nil
		InvenArmory:ClearRequestUpdater()
		collectgarbage()
	end)
	f.scrollframe = CreateFrame("ScrollFrame", "InvenArmoryFrameTalentScroll", f, "UIPanelScrollFrameTemplate")
	f.scrollchild = CreateFrame("Frame", "InvenArmoryFrameTalentScrollChild", f.scrollframe)
	f.scrollframe:SetScrollChild(f.scrollchild)
	f.scrollframe:SetWidth(198)
	f.scrollframe:SetHeight(300)
	f.scrollframe:SetPoint("TOPLEFT", f, "TOPLEFT", 70, -80)
	f.scrollframe:SetScript("OnMouseWheel", function(self, wheel)
		if GetNumCompanions then
			ScrollFrameTemplate_OnMouseWheel(self, wheel)	-- for WotLK
		else
			ScrollFrameTemplate_OnMouseWheel(wheel)		-- for BC
		end
	end)
	f.scrollchild:SetWidth(198)
	f.scrollchild:SetHeight(45)
	f.scrollchild:SetScale(0.9)
	for i = 1, 40 do
		talentIcons[i] = CreateFrame("Button", "InvenArmoryFrameTalent"..i, f.scrollchild, "TalentButtonTemplate")
		talentIcons[i]:SetID(i)
		talentIcons[i].texture = _G["InvenArmoryFrameTalent"..i.."IconTexture"]
		talentIcons[i].rank = _G["InvenArmoryFrameTalent"..i.."Rank"]
		talentIcons[i].rankBorder = _G["InvenArmoryFrameTalent"..i.."RankBorder"]
		talentIcons[i].slot = _G["InvenArmoryFrameTalent"..i.."Slot"]
		talentIcons[i]:SetScript("OnEnter", self.ButtonOnEnter)
		talentIcons[i]:SetScript("OnLeave", self.ButtonOnLeave)
		talentIcons[i]:SetScript("OnClick", self.ButtonOnClick)
		talentIcons[i]:EnableMouse(true)
		talentIcons[i]:Hide()
	end
	talentArrows, talentBranchs = {}, {}
	talentBranchs[0] = CreateFrame("Frame", nil, talentIcons[1])
	talentBranchs[0]:SetAllPoints(f.scrollchild)
	talentBranchs[0]:Show()
	talentArrows[0] = CreateFrame("Frame", nil, f.scrollchild)
	talentArrows[0]:SetAllPoints(f.scrollchild)
	talentArrows[0]:Show()
	f:Hide()
	collectgarbage()
end

function InvenArmory:TalentButtonOnClick()
	InvenArmory:SetTalent(self:GetID())
end

function InvenArmory:ButtonOnEnter()
	if self.link then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetHyperlink(self.link)
		GameTooltip:Show()
	elseif self.tooltip then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(self.tooltip)
		GameTooltip:Show()
	end
end

function InvenArmory:ButtonOnLeave()
	GameTooltip:Hide()
end

function InvenArmory:ButtonOnClick(button)
	if button == "LeftButton" and self.link then
		if strfind(self.link, "item:") then
			if BlueItemInfo2 and IsAltKeyDown() then
				BlueItemInfo2:AddFavoriteItem(tonumber(strmatch(self.link, "item:(%d+)")))
				return
			elseif BlueItemInfo2 and IsShiftKeyDown() and IsControlKeyDown() then
				BlueItemInfo2:AddMemo(tonumber(strmatch(self.link, "item:(%d+)")))
				return
			elseif IsModifiedClick("DRESSUP") then
				DressUpItemLink(self.link)
				return
			end
		end
		if IsModifiedClick("CHATLINK") then
			ChatFrameEditBox:Show()
			ChatFrameEditBox:Insert(self.link)
		else
			ChatFrame_OnHyperlinkShow(self.link, self.link)
		end
	end
end